.Dd Jun 8, 2007
.Dt drood 8
.Sh NAME
.Nm Drood
.Nd Minimal KQueue-Based HTTP/1.1 Server
.Sh SYNOPSIS
.Nm drood -r Ao server-root Ac Oo -i Ao interface Ac -p Ao port Ac -e Ao root-resource Ac -d Ao cgi-dir Ac -t Ao read-timeout Ac -u Ao user Ac -g Ao group Ac -q Ao backlog Ac -c -x -n Oc
.Sh DESCRIPTION
Drood is a lightweight, kqueue-based, connection-multiplexing, HTTP/1.1
server, intended as an alternative to larger, more complex web servers for
websites where the server requirements are simple.  Drood supports
name-based virtual hosting.
.Pp
Drood reads a client request, and spits out the requested resource, or an
error message, back at the client.  Drood does not perform content
negotiation, nor access authentication, nor does it provide any response
headers useful for caching server responses beyond the "Date" and
"Last-Modified" headers.
.Pp
The server recognizes the HEAD, GET, and POST request methods, but only the
following request headers are examined:  the "Host" header, the
"If-Modified-Since" header, the "Expect: 100 continue" header, and  the
"Connection:  close" header.  Drood does extract and pass on to CGI
programs some information from request headers, namely the content of a
"Cookie" header if one is present, the content of a "Content-Length" header
if one is present, and the content of a "User-Agent" header if one is
present.  If any one of these headers occurs more than once in a request,
only the last instance is used.
.Pp
By default, FreeBSD limits each process to having a maximum of 64
simultaneously open descriptors.  This places a limit on the number of
connections drood may multiplex.  The sysctl settings below, allow for a
generous number of connections.  These values can be set at the
command-line with the "sysctl" utility.  It is recommended that they be
enabled permanently by placing these lines in /etc/sysctl.conf on the host
machine.
.Bd -literal -offset left
kern.maxfiles=10000
kern.maxfilesperproc=10000
kern.kq_calloutmax=10000
.Ed
.Pp
Accept filters SHOULD NOT be enabled on the host machine because they
prevent drood from timing out inactive connections.  It is doubtful accept
filters provide any benefit at all to multiplexing servers.
.Pp
Drood writes its pid into /var/run/drood.pid, if it can (ie., it is started
as root), and may be stopped with a SIGTERM.  A rc.d script is provided and
installed in /usr/local/etc/rc.d/.  Add the following lines to
/etc/rc.conf to start drood on system boot-up.  Replace the items in
brackets with values appropriate for your system, and/or add or remove
options as suits your system.
.Bd -literal -offset left
drood_enable="YES"
drood_flags="-i <ip-addr> -r <server-root> -u <user> -g <group>"
.Ed
.Pp
Once your /etc/rc.conf is configured you may start, stop, or restart drood,
or determine if it is running with the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/drood start
/usr/local/etc/rc.d/drood stop
/usr/local/etc/rc.d/drood restart
/usr/local/etc/rc.d/drood status
.Ed
.Pp
These are the mappings from filename suffixes to MIME types which drood
recognizes out-of-the-box.  They are hard-coded into the server.  Any file
which cannot be identified by these mappings is identified by the server as
"text/plain; charset=utf-8".
.Bl -column -offset left "jpeg" "application/x-shockwave-flash"
.It Sy Suffix Ta Sy MIME Type
.It html Ta text/html; charset=utf-8
.It htm  Ta text/html; charset=utf-8
.It xml  Ta text/xml; charset-utf-8
.It txt  Ta text/plain; charset=utf-8
.It css  Ta text/css
.It ico  Ta image/vnd.microsoft.icon
.It png  Ta image/png
.It jpg  Ta image/jpeg
.It jpeg Ta image/jpeg
.It tiff Ta image/tiff
.It tif  Ta image/tiff
.It gif  Ta image/gif
.It tgz  Ta application/x-compressed
.It zip  Ta application/zip
.It ps   Ta application/postscript
.It eps  Ta application/postscript
.It gz   Ta application/x-gzip
.It tgz  Ta application/x-compressed
.It pdf  Ta application/pdf
.It swf  Ta application/x-shockwave-flash
.It js   Ta application/x-javascript
.It mov  Ta video/quicktime
.It qt   Ta video/quicktime
.It mpg  Ta video/mpeg
.It mp2  Ta video/mpeg
.It mpa  Ta video/mpeg
.It mpe  Ta video/mpeg
.It avi  Ta video/x-msvideo
.It asf  Ta video/x-ms-asf
.It asx  Ta video/x-ms-asf
.It asr  Ta video/x-ms-asf
.It flv  Ta video/x-flv
.It ra   Ta audio/x-pn-realaudio
.It ram  Ta audio/x-pn-realaudio
.It wav  Ta audio/x-wav
.It mp3  Ta audio/mpeg
.El
.Pp
The following options are recognized.  Only the -r option is required, but
you will want to include the -i option when you are starting the server for
production use.
.Bl -tag -width "-r"
.It -r
The -r option is mandatory and specifies the server root directory, which
drood will make its current working directory.
.Pp
The resources served-up by drood are determined by the "Host" header of
requests, thus implementing name-based virtual hosting.  For every hostname
you wish to be recognized by the server, an identically-named subdirectory
of the server root directory must be created.  The static files served up
by each virtual host are placed into the identically-named subdirectory.
Any or all of the subdirectories may be symbolic links.  One subdirectory
must be named "default".  This is the directory which will be used to
fulfill static requests when no "Host" header is supplied by a client.  The
"default" directory can and should be a symbolic link.  If you would like a
directory to be used for more than one hostname, you must create links for
the extra hostnames.  For example, I could have, in my server root
directory, the following subdirectories.  Note the symlink for the loopback
address.  Without it, drood will not be able to find resources for requests
from loopback connections.  You should also add a link for the IP address
of the server, if you wish to allow clients to connect using URIs with the
server's IP address instead of its hostname.
.Bd -literal -offset left
mammothcheese.ca     -> actual subdirectory
127.0.0.1            -> symlink to mammothcheese.ca
default              -> symlink to mammothcheese.ca
www.mammothcheese.ca -> symlink to mammothcheese.ca
.Ed
.Pp
Drood will reject any request whose host header contains a virgule "/",
or starts with "." in order to prevent clients from accessing resources
outside of the virtual host subdirectories.  If the -d option is present,
requests whose "Host" headers reference the directory specified by that
option, will be rejected as well.  Directory names are read once, at
startup.  If you change the directory structure in the server root
directory, then you must restart drood to have the new structure
recognized.  Drood does a linear search through its list of directories, so
it is not suited to truly massive virtual hosting, which a service provider
might employ.
.It -e
The root resource of all virtual hosts defaults to a static resource named
"index.html", which itself may be a symbolic link.  This may be changed with
the -e option.  Note that this changes the root resource of all virtual hosts
served-up by drood.  You may, with the -e option, specify a dynamic resource
as the root resource.  If you do, you must be sure to include the -d option
to tell the server where CGI programs live.  Eg.,
.Bd -literal -offset left
drood -r /usr/local/www/drood -d cgi-bin -e '/cgi-bin/index.cgi'
.Ed
.Pp
Since all virtual hosts share the CGI directory, there can only be one
dynamic root resource for all virtual hosts, and it must use the
SERVER_NAME environment variable to determine the host on whose behalf it
is being executed.
.It -d
The -d option, if present, specifies the subdirectory of the server root
directory which contains CGI programs.  Only executables in this directory
can be run.  Drood refuses to execute symbolic links.  All virtual hosts
share the dynamic directory.  If CGI programs wish to know what hostname
they have been invoked for, they must examine the SERVER_NAME environment
variable.  If no "Host" header was supplied with the request, then
SERVER_NAME will be set to the string "default".  If the -d option is not
present, executables cannot be run.  Requests for dynamic resources must
include the dynamic path in their URIs, because this is how drood
differentiates static requests from dynamic requests.  Eg., if -d is
"cgi-bin", then
.Bd -literal -offset left
http://host.domain/cgi-bin/foobar.cgi
.Ed
.Pp
is a request to run foobar.cgi.
.Pp
CGI programs need only precede their response bodies with a "Content-Type"
header and a blank line.  Drood will chunk response data for CGI programs
invoked over persistent connections.  The CGI program's response header
will be supplemented with a response status line, a "Date" header if
one is missing, a "Server" header, and if the CGI response header does not
include a "Content-Length" header and the connection is persistent, a
"Transfer-Encoding: chunked" header.  If a "Location" header is provided by
the CGI program, then drood with create a "302 Found" response to send to
the client, which will cause the client to redirect itself to the URI
specified in the "Location" header.  Otherwise, drood will compose a "200
OK" response status line.
.Pp
The stderr of the CGI program is discarded. For testing purposes run drood
with -x option and the CGI stderr will be connected to the terminal from
which drood is running.  For production use, CGI programs must send
diagnostic output to files, or use syslog(3).
.Pp
Only the following environment variables are set for CGI programs:
.Pp
REMOTE_ADDR, REQUEST_METHOD, SCRIPT_NAME, QUERY_STRING, HTTP_USER_AGENT,
HTTP_COOKIE, SERVER_SOFTWARE, SERVER_NAME, GATEWAY_INTERFACE, and
CONTENT_LENGTH.
.Pp
.It -c
The -c option, if present, makes drood chroot(2) to the server root
directory.  Note that to run CGI programs from a chrooted drood, they must
either be statically compiled, or you must create and populate a minimal
chroot filesystem containing the shared libraries and executables required
by the CGI programs you intend to run.
.It -p
The -p option specifies the port to listen on.  This defaults to 80 if not
specified.
.It -i
The -i option specifies the interface to listen on.  This defaults to the
loopback device if not specified:  127.0.0.1.
.It -u
.It -g
Drood must be started as root in order to bind to port 80, but it tries to
setuid to user "nobody" and setgid to group "nobody" after binding the
listening socket.  If it is unsuccessful in either attempt, the server will
exit.  The -u and the -g options may be used to specify a different user
and group.  If you use the -p option to make drood listen on a port higher
than port 1023, then drood can be started as any user, but it will not be
able to change to user and group "nobody".  In this case, you must provide
the -u and -g options to specify the user and group that drood will begin
running as and stay running as, or the server will exit.
.It -t
When reading a request header from a client, drood drops the connection if
it cannot read the complete request header within five seconds.  The -t
option, if present, specifies a different read timeout value, in seconds.
The value must be in the range 1-60 seconds.
.It -q
The -q option, if present, specifies the backlog of client connections
queued by the OS kernel for the server to subsequently service.  This value
defaults to 100.  Connections arriving when the queue is full will be
dropped.
.It -x
The -x option, if present, prevents drood from becoming a daemon.  It will
then run in the foreground of the terminal where it was started, and may be
stopped with signals (ie., Control-C).  The server also will not write its
pid to /var/run/drood.pid when the -x option is used.
.It -n
The -n option turns off transaction logging.  System errors are always
reported and may not be turned off.  Drood logs transactions and system
errors via syslog(3).  A typical transaction log entry is below.  The
single line from the access log has been folded at whitespace to make it
easier to read.  Everything up to the third colon is the prefix added by
syslogd.  What follows after is the data for one client request, consisting
of the client IP address in brackets, the client user-agent string enclosed
in braces, the request method in parenthesis, and the request URI delimited
by angle brackets, each component separated by single space characters.
The purpose of delimiting each component with its own delimiter characters,
is to make the writing of automated tools to extract data from log files
easier.
.Bd -literal -offset left
Jun 13 11:30:34 localhost drood[2517]: [127.0.0.1]
.br
{Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.8.1.4)
.br
Gecko/20070609 Firefox/2.0.0.4} (GET) <http://127.0.0.1/turn3.png>
.Ed
.Pp
Where log message end up is determined by /etc/syslog.conf, most likely in
/var/log/messages.  Add the following three lines to /etc/syslog.conf if
you want drood messages to be written to their own files:
.Bd -literal -offset left
!drood
*.>=notice[4 tab characters]/var/log/drood-error.log
*.<notice[4 tab characters]/var/log/drood-access.log
.Ed
.Pp
Then to get syslogd to start using the logs, do:
.Bd -literal -offset left
touch /var/log/drood-error.log /var/log/drood-access.log
kill -HUP `cat /var/run/syslog.pid`
.Ed
.El
.Sh AUTHORS
.An James Bailie Aq jimmy@mammothcheese.ca
.br
http://www.mammothcheese.ca
